#!/usr/bin/env python
"""
This module implements a minimal set of funtions which allow to perform
the stochastic analysis of real-time systems in a semi-manual way.
For more information see technical report available at
http://www.atc.uniovi.es/research/SNSAUE07.pdf
"""

# CORE SET OF FUNCTIONS
# ==========================================================
def convolve(f,g):
  """
  Implements the convolution of two probability functions
  """
  r=[0.0]*(len(f)+len(g)-1)  # Make up space for result
  for i in range(len(f)):
    for j in range(len(g)):
      r[i+j]+=f[i]*g[j]
  return r

def shrink(f, c):
    """
    Implements the 'shrinking' of a probability function in the amount c
    """
    return [sum(f[:c+1])]+f[c+1:]

def AF(r,t,c):
  """
  Implements the 'Addition From' operation, which provides the response
  time of Job_a, including the interference of Job_b, being
    r: provisional response time of Job_a
    t: arrival time of Job_a, with respect of arrival time of Job_b
    c: execution time of Job_b
  """
  if t>len(r): return r  # No interference is possible
  return r[:t+1]+convolve(r[t+1:], c)

# Utility functions
# ==========================================================
def difference(f,g):
    """
    Computes the quadratic difference between two probability functions
    """
    m=min(len(f), len(g))
    M=max(len(f), len(g))
    if (M==len(f)): longest=f
    else: longest=g
    r=0.0
    for i in range(m): r=r+abs(f[i]-g[i])
    for i in range(m,M):
        r=r+longest[i]
    return r

def average(PFs):
    """
    Given a list of probability functions, it returns the average
    """
    maxsize=max(map(len, PFs))
    for pf in PFs:
      pf.extend([0]*(maxsize-len(pf)))
    r=[]
    for i in range(maxsize):
      r.append(sum([x[i] for x in  PFs]))
    return [x/len(PFs) for x in r]

def PDF(pf):
    """
    Given a probability function (pf), it returns the cumulative
    distribution function (pdf).
    """
    pdf=[]; acum=0
    for v in pf:
        acum+=v
        pdf.append(acum)
    return pdf

def PF(d):
    """
    Given a PF implemented in a dictionary (pairs of time:probability)
    returns a list (array) of probabilities indexed by time. For
    example, the dictionary {0:0.2, 7: 0.3, 15:0.5} would produce
    the list [0.2, 0, 0, 0, 0, 0, 0, 0.3, 0, 0, 0, 0, 0, 0, 0, 0.5]
    with 16 elements.
    The first form is more suitable for input disperse functions, but
    the second is more suitable for the analysis.
    """
    m=max(d.keys())
    lista=[]
    for i in range(0,m+1):
        if d.has_key(i): lista.append(d[i])
        else: lista.append(0)
    return lista

# Input/Output functions
# ===========================================================
def PF_load(filename):
    """
    Given the name of a file, this functions reads it an returns a
    list (array) in which the index is the time and the value
    the probability.
    The file is expected to contain in each line a pair 'time probability'
    separated by tab ('\t')
    """
    
    d={}
    fil=open(filename, "r")
    for l in fil:
        c=l.split('\t')
        if len(c)==2:
          d[int(c[0])]=float(c[1])
    return PF(d)

def PF_save(pf, filename, decimal_places=15):
    """
    Saves the probability function pf in a file, in a format suitable
    for plotting with gnuplot or tikz.
    """
    format="%%10i\t%%2.%df" % decimal_places
    fil=open(filename, "w")
    for i in range(len(pf)):
        print >> fil, format % (i, pf[i])

# END of module
# ===========================================================
def main():
  """
  Called when the module is executed instead of included, it only
  prints an error message and exits.
  """
  print "This module is for being included, not executed."
 
if __name__ == "__main__":
    main()
