# -*- encoding: utf8 -*-
# This code shows how the analysis of an EDF system can be performed.
# It solves the example in [SNSAUES07]. This implementation is not aimed
# to efficiency nor elegancy. It is written in such a way that mimics
# the algorithm detailed in Fig.3 of [SNSAUES07]

# System parameters
#
#            |  Task 1   | Task 2   |
#      Phase |  1 = 20   | 2 = 50   |
#     Period |  T1 = 40  | T2 = 60  |
#   Deadline |  D1 = 50  | D2 = 90  |
# Exec. time |  C1 = C   | C2 = C   |

from analysis import PF, PDF, shrink, convolve, AF,\
                     difference, average, PF_save

# Execution time probabilities (both tasks have the same distribution)
C=PF({10: 0.1,      20: 0.4,      21: 0.2,      22: 0.2,      50: 0.1})

################################################################
# CALCULATION OF THE STEADY STATE SYSTEM BACKLOG
################################################################

# Set initial backlog equal to zero with probability 1
W0=PF({ 0:1 })

# Advance to instant 20, in which the first job arrives
W20=shrink(W0,20)

# Iterative procedure
iteration=1; diff=1
while(diff>0.00001):
  W50  = shrink(convolve(C, W20),  50-20)
  W60  = shrink(convolve(C, W50),  60-50)
  W100 = shrink(convolve(C, W60),  100-60)
  W110 = shrink(convolve(C, W100), 110-100)
  W140 = shrink(convolve(C, W110), 140-110)
  diff = difference(W140, W20)
  iteration=iteration+1
  # Copy the backlog at the end of the previous hyperperiod
  # as the backlog at the beginning of the new hyperperiod
  W20=W140
  print "# Iteration %d. diff=%2.10f. Backlog size=%d" %\
        (iteration, diff, len(W140))

################################################################
# CALCULATION OF THE STEADY STATE BACKLOG AFFECTING JOB 1
################################################################
W50 =shrink(convolve(C, W20),  50-20)
W60 =shrink(convolve(C, W50),  60-50)
W100=shrink(convolve(C, W60),  100-60)
W110=shrink(convolve(C, W100), 110-100)
W140=shrink(W110, 140-110) # No convolve in this step
                           # this way Job5 (at 110) is excluded
W20=W140

# CALCULATION OF THE RESPONSE TIME OF Job1
################################################################
Rj1=convolve(W20, C)

# CALCULATION OF THE RESPONSE TIME OF Job2
################################################################
Rj2=convolve(W50, C)
Rj2=AF(Rj2, 60-50, C)   # Interference of job3

# CALCULATION OF THE RESPONSE TIME OF Job3
################################################################
# The backlog affecting Job3 must be recomputed, since W60
# includes Job2, with lower priority
W60 =shrink(W50,  60-50)     # Do not include Job2
Rj3=convolve(W60, C)

# CALCULATION OF THE RESPONSE TIME OF Job4
################################################################
Rj4=convolve(W100, C)

# CALCULATION OF THE RESPONSE TIME OF Job5
################################################################
Rj5=convolve(W110, C)
Rj5=AF(Rj5, 140-110, C)  # Interference of job1 in next hyperperiod

# CALCULATION OF THE RESPONSE TIME OF TASK1
################################################################
Rt1=average([Rj1, Rj3, Rj4])

# CALCULATION OF THE RESPONSE TIME OF TASK2
################################################################
Rt2=average([Rj2, Rj5])

################################################################
# CALCULATION OF PROBABILITIES
################################################################
D1=50  # Deadlines
D2=90  #
print "Probability of deadline misses for Task1 = %2.10f" %\
      (1-PDF(Rt1)[D1])
print "Probability of deadline misses for Task2 = %2.10f" %\
      (1-PDF(Rt2)[D2])
# Save cumulative distributions for plotting (only first 201 points)
PF_save(PDF(Rt1[:201]), "Task-1-pdf.dat")
PF_save(PDF(Rt2[:201]), "Task-2-pdf.dat")
